@@ -12,6 +12,7 @@ import sys
 import itertools
 
 from zope.interface import implementer
+from constantly import NamedConstant, Names
 
 skipSSL = None
 skipSNI = None
@@ -28,8 +29,6 @@ else:
     from OpenSSL import SSL
     from OpenSSL.crypto import PKey, X509
     from OpenSSL.crypto import TYPE_RSA, FILETYPE_PEM
-    if getattr(SSL.Context, "set_tlsext_servername_callback", None) is None:
-        skipSNI = "PyOpenSSL 0.13 or greater required for SNI support."
 
     try:
         ctx = SSL.Context(SSL.SSLv23_METHOD)
@@ -52,21 +51,22 @@ from twisted.test.iosim import connectedServerAndClient
 
 from twisted.internet.error import ConnectionClosed
 from twisted.python.compat import nativeString, _PY3
-from twisted.python.constants import NamedConstant, Names
 from twisted.python.filepath import FilePath
+from twisted.python.modules import getModule
 
 from twisted.trial import unittest, util
 from twisted.internet import protocol, defer, reactor
 
 from twisted.internet.error import CertificateError, ConnectionLost
 from twisted.internet import interfaces
-from twisted.python.versions import Version
+from incremental import Version
 
 if not skipSSL:
     from twisted.internet.ssl import platformTrust, VerificationError
     from twisted.internet import _sslverify as sslverify
     from twisted.protocols.tls import TLSMemoryBIOFactory
 
+
 # A couple of static PEM-format certificates to be used by various tests.
 A_HOST_CERTIFICATE_PEM = """
 -----BEGIN CERTIFICATE-----
@@ -110,6 +110,8 @@ A_PEER_CERTIFICATE_PEM = """
 -----END CERTIFICATE-----
 """
 
+A_KEYPAIR = getModule(__name__).filePath.sibling('server.pem').getContent()
+
 
 
 class DummyOpenSSL(object):
@@ -379,35 +381,6 @@ class WritingProtocol(protocol.Protocol):
 
 
 
-if not skipSSL:
-    class ALPNOnlyOptions(sslverify.OpenSSLCertificateOptions):
-        """
-        An OpenSSLCertificateOptions subclass that only sets ALPN.
-        """
-        def _setUpNextProtocolMechanisms(self, ctx):
-            """
-            Only set up ALPN.
-            """
-            ctx.set_alpn_select_callback(self._protoSelectCallback)
-            ctx.set_alpn_protos(self._acceptableProtocols)
-
-
-    class NPNOnlyOptions(sslverify.OpenSSLCertificateOptions):
-        """
-        An OpenSSLCertificateOptions subclass that only sets NPN.
-        """
-        def _setUpNextProtocolMechanisms(self, ctx):
-            """
-            Only set NPN.
-            """
-            def npnAdvertiseCallback(conn):
-                return self._acceptableProtocols
-
-            ctx.set_npn_advertise_callback(npnAdvertiseCallback)
-            ctx.set_npn_select_callback(self._protoSelectCallback)
-
-
-
 class FakeContext(object):
     """
     Introspectable fake of an C{OpenSSL.SSL.Context}.
@@ -419,7 +392,7 @@ class FakeContext(object):
 
     @ivar _method: See C{method} parameter of L{__init__}.
 
-    @ivar _options: C{int} of C{OR}ed values from calls of L{set_options}.
+    @ivar _options: L{int} of C{OR}ed values from calls of L{set_options}.
 
     @ivar _certificate: Set by L{use_certificate}.
 
@@ -431,7 +404,7 @@ class FakeContext(object):
 
     @ivar _sessionID: Set by L{set_session_id}.
 
-    @ivar _extraCertChain: Accumulated C{list} of all extra certificates added
+    @ivar _extraCertChain: Accumulated L{list} of all extra certificates added
         by L{add_extra_chain_cert}.
 
     @ivar _cipherList: Set by L{set_cipher_list}.
@@ -774,7 +747,7 @@ class OpenSSLOptionsTests(unittest.TestCase):
         )
         opts._contextFactory = FakeContext
         ctx = opts.getContext()
-        self.assertEqual(opts._cipherString, ctx._cipherList)
+        self.assertEqual(opts._cipherString.encode('ascii'), ctx._cipherList)
 
 
     def test_givesMeaningfulErrorMessageIfNoCipherMatches(self):
@@ -808,7 +781,7 @@ class OpenSSLOptionsTests(unittest.TestCase):
         )
         opts._contextFactory = FakeContext
         ctx = opts.getContext()
-        self.assertEqual(u'sentinel', ctx._cipherList)
+        self.assertEqual(b'sentinel', ctx._cipherList)
 
 
     def test_basicSecurityOptionsAreSet(self):
@@ -877,7 +850,7 @@ class OpenSSLOptionsTests(unittest.TestCase):
             privateKey=self.sKey,
             certificate=self.sCert,
         )
-        self.assertIs(None, opts._ecCurve)
+        self.assertIsNone(opts._ecCurve)
 
 
     def test_ecNeverBreaksGetContext(self):
@@ -925,7 +898,7 @@ class OpenSSLOptionsTests(unittest.TestCase):
                 sslverify.DN(CN=b'a', OU=b'hello'),
                 sslverify.DistinguishedName(commonName=b'a',
                                             organizationalUnitName=b'hello'))
-        self.assertNotEquals(
+        self.assertNotEqual(
                 sslverify.DN(CN=b'a', OU=b'hello'),
                 sslverify.DN(CN=b'a', OU=b'hello', emailAddress=b'xxx'))
         dn = sslverify.DN(CN=b'abcdefg')
@@ -1034,7 +1007,7 @@ class OpenSSLOptionsTests(unittest.TestCase):
         firstOpts = sslverify.OpenSSLCertificateOptions(
             privateKey=self.sKey,
             certificate=self.sCert,
-            method=SSL.SSLv3_METHOD,
+            method=SSL.SSLv23_METHOD,
             verify=True,
             caCerts=[self.sCert],
             verifyDepth=2,
@@ -1045,8 +1018,8 @@ class OpenSSLOptionsTests(unittest.TestCase):
             fixBrokenPeers=True,
             enableSessionTickets=True)
         context = firstOpts.getContext()
-        self.assertIdentical(context, firstOpts._context)
-        self.assertNotIdentical(context, None)
+        self.assertIs(context, firstOpts._context)
+        self.assertIsNotNone(context)
         state = firstOpts.__getstate__()
         self.assertNotIn("_context", state)
 
@@ -1054,16 +1027,16 @@ class OpenSSLOptionsTests(unittest.TestCase):
         opts.__setstate__(state)
         self.assertEqual(opts.privateKey, self.sKey)
         self.assertEqual(opts.certificate, self.sCert)
-        self.assertEqual(opts.method, SSL.SSLv3_METHOD)
-        self.assertEqual(opts.verify, True)
+        self.assertEqual(opts.method, SSL.SSLv23_METHOD)
+        self.assertTrue(opts.verify)
         self.assertEqual(opts.caCerts, [self.sCert])
         self.assertEqual(opts.verifyDepth, 2)
-        self.assertEqual(opts.requireCertificate, False)
-        self.assertEqual(opts.verifyOnce, False)
-        self.assertEqual(opts.enableSingleUseKeys, False)
-        self.assertEqual(opts.enableSessions, False)
-        self.assertEqual(opts.fixBrokenPeers, True)
-        self.assertEqual(opts.enableSessionTickets, True)
+        self.assertFalse(opts.requireCertificate)
+        self.assertFalse(opts.verifyOnce)
+        self.assertFalse(opts.enableSingleUseKeys)
+        self.assertFalse(opts.enableSessions)
+        self.assertTrue(opts.fixBrokenPeers)
+        self.assertTrue(opts.enableSessionTickets)
 
     test_certificateOptionsSerialization.suppress = [
         util.suppress(category = DeprecationWarning,
@@ -1125,8 +1098,8 @@ class OpenSSLOptionsTests(unittest.TestCase):
 
         def afterLost(result):
             ((cSuccess, cResult), (sSuccess, sResult)) = result
-            self.failIf(cSuccess)
-            self.failIf(sSuccess)
+            self.assertFalse(cSuccess)
+            self.assertFalse(sSuccess)
             # Win32 fails to report the SSL Error, and report a connection lost
             # instead: there is a race condition so that's not totally
             # surprising (see ticket #2877 in the tracker)
@@ -1154,8 +1127,8 @@ class OpenSSLOptionsTests(unittest.TestCase):
                                consumeErrors=True)
         def afterLost(result):
             ((cSuccess, cResult), (sSuccess, sResult)) = result
-            self.failIf(cSuccess)
-            self.failIf(sSuccess)
+            self.assertFalse(cSuccess)
+            self.assertFalse(sSuccess)
 
         return d.addCallback(afterLost)
 
@@ -1412,7 +1385,7 @@ class TrustRootTests(unittest.TestCase):
             chainedCertFile=pathContainingDumpOf(self, serverCert),
         )
         pump.flush()
-        self.assertIs(cProto.wrappedProtocol.lostReason, None)
+        self.assertIsNone(cProto.wrappedProtocol.lostReason)
         self.assertEqual(cProto.wrappedProtocol.data,
                          sProto.wrappedProtocol.greeting)
 
@@ -1451,7 +1424,7 @@ class ServiceIdentityTests(unittest.SynchronousTestCase):
         @param serverContextSetup: a 1-argument callable invoked with the
             L{OpenSSL.SSL.Context} after it's produced.
         @type serverContextSetup: L{callable} taking L{OpenSSL.SSL.Context}
-            returning L{NoneType}.
+            returning L{None}.
 
         @param validCertificate: Is the server's certificate valid?  L{True} if
             so, L{False} otherwise.
@@ -1609,8 +1582,8 @@ class ServiceIdentityTests(unittest.SynchronousTestCase):
 
         cErr = cProto.wrappedProtocol.lostReason
         sErr = sProto.wrappedProtocol.lostReason
-        self.assertIdentical(cErr, None)
-        self.assertIdentical(sErr, None)
+        self.assertIsNone(cErr)
+        self.assertIsNone(sErr)
 
 
     def test_validHostnameInvalidCertificate(self):
@@ -1672,8 +1645,8 @@ class ServiceIdentityTests(unittest.SynchronousTestCase):
 
         cErr = cProto.wrappedProtocol.lostReason
         sErr = sProto.wrappedProtocol.lostReason
-        self.assertIdentical(cErr, None)
-        self.assertIdentical(sErr, None)
+        self.assertIsNone(cErr)
+        self.assertIsNone(sErr)
 
 
     def test_clientPresentsCertificate(self):
@@ -1695,8 +1668,8 @@ class ServiceIdentityTests(unittest.SynchronousTestCase):
 
         cErr = cProto.wrappedProtocol.lostReason
         sErr = sProto.wrappedProtocol.lostReason
-        self.assertIdentical(cErr, None)
-        self.assertIdentical(sErr, None)
+        self.assertIsNone(cErr)
+        self.assertIsNone(sErr)
 
 
     def test_clientPresentsBadCertificate(self):
@@ -1768,8 +1741,8 @@ class ServiceIdentityTests(unittest.SynchronousTestCase):
 
         cErr = cProto.wrappedProtocol.lostReason
         sErr = sProto.wrappedProtocol.lostReason
-        self.assertIdentical(cErr, None)
-        self.assertIdentical(sErr, None)
+        self.assertIsNone(cErr)
+        self.assertIsNone(sErr)
 
     if skipSNI is not None:
         test_hostnameEncoding.skip = skipSNI
@@ -1794,7 +1767,7 @@ class ServiceIdentityTests(unittest.SynchronousTestCase):
                 cert.get_subject().commonName = name
                 return cert
         conn = Connection()
-        self.assertIdentical(
+        self.assertIs(
             sslverify.simpleVerifyHostname(conn, u'something.example.com'),
             None
         )
@@ -1822,7 +1795,7 @@ class ServiceIdentityTests(unittest.SynchronousTestCase):
         sErr = sProto.wrappedProtocol.lostReason.value
 
         self.assertIsInstance(cErr, ZeroDivisionError)
-        self.assertIsInstance(sErr, ConnectionClosed)
+        self.assertIsInstance(sErr, (ConnectionClosed, SSL.Error))
         errors = self.flushLoggedErrors(ZeroDivisionError)
         self.assertTrue(errors)
 
@@ -1838,7 +1811,7 @@ def negotiateProtocol(serverProtocols,
     @param clientProtocols: The protocols the client is willing to negotiate.
     @param clientOptions: The type of C{OpenSSLCertificateOptions} class to
         use for the client. Defaults to C{OpenSSLCertificateOptions}.
-    @return: A C{typle} of: the negotiated protocol and the reason the
+    @return: A L{tuple} of the negotiated protocol and the reason the
         connection was lost.
     """
     caCertificate, serverCertificate = certificatesForAuthorityAndServer()
@@ -1895,7 +1868,7 @@ class NPNOrALPNTests(unittest.TestCase):
             serverProtocols=protocols,
         )
         self.assertEqual(negotiatedProtocol, b'h2')
-        self.assertEqual(lostReason, None)
+        self.assertIsNone(lostReason)
 
 
     def test_NPNAndALPNDifferent(self):
@@ -1910,7 +1883,7 @@ class NPNOrALPNTests(unittest.TestCase):
             serverProtocols=serverProtocols,
         )
         self.assertEqual(negotiatedProtocol, b'http/1.1')
-        self.assertEqual(lostReason, None)
+        self.assertIsNone(lostReason)
 
 
     def test_NPNAndALPNNoAdvertise(self):
@@ -1923,8 +1896,8 @@ class NPNOrALPNTests(unittest.TestCase):
             clientProtocols=protocols,
             serverProtocols=[],
         )
-        self.assertEqual(negotiatedProtocol, None)
-        self.assertEqual(lostReason, None)
+        self.assertIsNone(negotiatedProtocol)
+        self.assertIsNone(lostReason)
 
 
     def test_NPNAndALPNNoOverlap(self):
@@ -1938,25 +1911,10 @@ class NPNOrALPNTests(unittest.TestCase):
             serverProtocols=clientProtocols,
             clientProtocols=serverProtocols,
         )
-        self.assertEqual(negotiatedProtocol, None)
+        self.assertIsNone(negotiatedProtocol)
         self.assertEqual(lostReason.type, SSL.Error)
 
 
-    def test_NPNRespectsClientPreference(self):
-        """
-        When NPN is used, the client's protocol preference is preferred.
-        """
-        serverProtocols = [b'http/1.1', b'h2']
-        clientProtocols = [b'h2', b'http/1.1']
-        negotiatedProtocol, lostReason = negotiateProtocol(
-            clientProtocols=clientProtocols,
-            serverProtocols=serverProtocols,
-            clientOptions=NPNOnlyOptions
-        )
-        self.assertEqual(negotiatedProtocol, b'h2')
-        self.assertEqual(lostReason, None)
-
-
 
 class ALPNTests(unittest.TestCase):
     """
@@ -1986,21 +1944,6 @@ class ALPNTests(unittest.TestCase):
         )
 
 
-    def test_ALPNRespectsServerPreference(self):
-        """
-        When ALPN is used, the server's protocol preference is preferred.
-        """
-        serverProtocols = [b'http/1.1', b'h2']
-        clientProtocols = [b'h2', b'http/1.1']
-        negotiatedProtocol, lostReason = negotiateProtocol(
-            clientProtocols=clientProtocols,
-            serverProtocols=serverProtocols,
-            clientOptions=ALPNOnlyOptions
-        )
-        self.assertEqual(negotiatedProtocol, b'http/1.1')
-        self.assertEqual(lostReason, None)
-
-
 
 class NPNAndALPNAbsentTests(unittest.TestCase):
     """
@@ -2040,7 +1983,7 @@ class NPNAndALPNAbsentTests(unittest.TestCase):
 
     def test_NegotiatedProtocolReturnsNone(self):
         """
-        negotiatedProtocol return C{None} even when NPN/ALPN aren't supported.
+        negotiatedProtocol return L{None} even when NPN/ALPN aren't supported.
         This works because, as neither are supported, negotiation isn't even
         attempted.
         """
@@ -2050,8 +1993,8 @@ class NPNAndALPNAbsentTests(unittest.TestCase):
             clientProtocols=clientProtocols,
             serverProtocols=serverProtocols,
         )
-        self.assertEqual(negotiatedProtocol, None)
-        self.assertEqual(lostReason, None)
+        self.assertIsNone(negotiatedProtocol)
+        self.assertIsNone(lostReason)
 
 
 
@@ -2097,7 +2040,7 @@ class ConstructorsTests(unittest.TestCase):
         x = self.assertRaises(CertificateError,
                               sslverify.Certificate.peerFromTransport,
                               _NotSSLTransport())
-        self.failUnless(str(x).startswith("non-TLS"))
+        self.assertTrue(str(x).startswith("non-TLS"))
 
 
     def test_peerFromBlankSSLTransport(self):
@@ -2108,7 +2051,7 @@ class ConstructorsTests(unittest.TestCase):
         x = self.assertRaises(CertificateError,
                               sslverify.Certificate.peerFromTransport,
                               _MaybeSSLTransport())
-        self.failUnless(str(x).startswith("TLS"))
+        self.assertTrue(str(x).startswith("TLS"))
 
 
     def test_hostFromNonSSLTransport(self):
@@ -2119,7 +2062,7 @@ class ConstructorsTests(unittest.TestCase):
         x = self.assertRaises(CertificateError,
                               sslverify.Certificate.hostFromTransport,
                               _NotSSLTransport())
-        self.failUnless(str(x).startswith("non-TLS"))
+        self.assertTrue(str(x).startswith("non-TLS"))
 
 
     def test_hostFromBlankSSLTransport(self):
@@ -2130,7 +2073,7 @@ class ConstructorsTests(unittest.TestCase):
         x = self.assertRaises(CertificateError,
                               sslverify.Certificate.hostFromTransport,
                               _MaybeSSLTransport())
-        self.failUnless(str(x).startswith("TLS"))
+        self.assertTrue(str(x).startswith("TLS"))
 
 
     def test_hostFromSSLTransport(self):
@@ -2156,6 +2099,140 @@ class ConstructorsTests(unittest.TestCase):
 
 
 
+class MultipleCertificateTrustRootTests(unittest.TestCase):
+    """
+    Test the behavior of the trustRootFromCertificates() API call.
+    """
+
+    if skipSSL:
+        skip = skipSSL
+
+
+    def test_trustRootFromCertificatesPrivatePublic(self):
+        """
+        L{trustRootFromCertificates} accepts either a L{sslverify.Certificate}
+        or a L{sslverify.PrivateCertificate} instance.
+        """
+        privateCert = sslverify.PrivateCertificate.loadPEM(A_KEYPAIR)
+        cert = sslverify.Certificate.loadPEM(A_HOST_CERTIFICATE_PEM)
+
+        mt = sslverify.trustRootFromCertificates([privateCert, cert])
+
+        # Verify that the returned object acts correctly when used as a
+        # trustRoot= param to optionsForClientTLS.
+        sProto, cProto, pump = loopbackTLSConnectionInMemory(
+            trustRoot=mt,
+            privateKey=privateCert.privateKey.original,
+            serverCertificate=privateCert.original,
+        )
+
+        # This connection should succeed
+        self.assertEqual(cProto.wrappedProtocol.data, b'greetings!')
+        self.assertIsNone(cProto.wrappedProtocol.lostReason)
+
+
+    def test_trustRootSelfSignedServerCertificate(self):
+        """
+        L{trustRootFromCertificates} called with a single self-signed
+        certificate will cause L{optionsForClientTLS} to accept client
+        connections to a server with that certificate.
+        """
+        key, cert = makeCertificate(O=b"Server Test Certificate", CN=b"server")
+        selfSigned = sslverify.PrivateCertificate.fromCertificateAndKeyPair(
+            sslverify.Certificate(cert),
+            sslverify.KeyPair(key),
+        )
+
+        trust = sslverify.trustRootFromCertificates([selfSigned])
+
+        # Since we trust this exact certificate, connections to this server
+        # should succeed.
+        sProto, cProto, pump = loopbackTLSConnectionInMemory(
+            trustRoot=trust,
+            privateKey=selfSigned.privateKey.original,
+            serverCertificate=selfSigned.original,
+        )
+        self.assertEqual(cProto.wrappedProtocol.data, b'greetings!')
+        self.assertIsNone(cProto.wrappedProtocol.lostReason)
+
+
+    def test_trustRootCertificateAuthorityTrustsConnection(self):
+        """
+        L{trustRootFromCertificates} called with certificate A will cause
+        L{optionsForClientTLS} to accept client connections to a server with
+        certificate B where B is signed by A.
+        """
+        caCert, serverCert = certificatesForAuthorityAndServer()
+
+        trust = sslverify.trustRootFromCertificates([caCert])
+
+        # Since we've listed the CA's certificate as a trusted cert, a
+        # connection to the server certificate it signed should succeed.
+        sProto, cProto, pump = loopbackTLSConnectionInMemory(
+            trustRoot=trust,
+            privateKey=serverCert.privateKey.original,
+            serverCertificate=serverCert.original,
+        )
+        self.assertEqual(cProto.wrappedProtocol.data, b'greetings!')
+        self.assertIsNone(cProto.wrappedProtocol.lostReason)
+
+
+    def test_trustRootFromCertificatesUntrusted(self):
+        """
+        L{trustRootFromCertificates} called with certificate A will cause
+        L{optionsForClientTLS} to disallow any connections to a server with
+        certificate B where B is not signed by A.
+        """
+        key, cert = makeCertificate(O=b"Server Test Certificate", CN=b"server")
+        serverCert = sslverify.PrivateCertificate.fromCertificateAndKeyPair(
+            sslverify.Certificate(cert),
+            sslverify.KeyPair(key),
+        )
+        untrustedCert = sslverify.Certificate(
+            makeCertificate(O=b"CA Test Certificate", CN=b"unknown CA")[1]
+        )
+
+        trust = sslverify.trustRootFromCertificates([untrustedCert])
+
+        # Since we only trust 'untrustedCert' which has not signed our
+        # server's cert, we should reject this connection
+        sProto, cProto, pump = loopbackTLSConnectionInMemory(
+            trustRoot=trust,
+            privateKey=serverCert.privateKey.original,
+            serverCertificate=serverCert.original,
+        )
+
+        # This connection should fail, so no data was received.
+        self.assertEqual(cProto.wrappedProtocol.data, b'')
+
+        # It was an L{SSL.Error}.
+        self.assertEqual(cProto.wrappedProtocol.lostReason.type, SSL.Error)
+
+        # Some combination of OpenSSL and PyOpenSSL is bad at reporting errors.
+        err = cProto.wrappedProtocol.lostReason.value
+        self.assertEqual(err.args[0][0][2], 'tlsv1 alert unknown ca')
+
+
+    def test_trustRootFromCertificatesOpenSSLObjects(self):
+        """
+        L{trustRootFromCertificates} rejects any L{OpenSSL.crypto.X509}
+        instances in the list passed to it.
+        """
+        private = sslverify.PrivateCertificate.loadPEM(A_KEYPAIR)
+        certX509 = private.original
+
+        exception = self.assertRaises(
+            TypeError,
+            sslverify.trustRootFromCertificates, [certX509],
+        )
+        self.assertEqual(
+            "certificates items must be twisted.internet.ssl.CertBase "
+            "instances",
+            exception.args[0],
+        )
+
+
+
 class OpenSSLCipherTests(unittest.TestCase):
     """
     Tests for twisted.internet._sslverify.OpenSSLCipher.
